@@ -7,19 +7,35 @@ use XML::Parser::Lite::Tree;
 use Flickr::API::Request;
 use Flickr::API::Response;
 use Digest::MD5 qw(md5_hex);
+use Scalar::Util qw(blessed);
+use Encode qw(encode_utf8);
 
 our @ISA = qw(LWP::UserAgent);
 
-our $VERSION = '1.01';
+our $VERSION = '1.08';
 
 sub new {
 	my $class = shift;
 	my $options = shift;
-	my $self = new LWP::UserAgent;
+
+	my $self;
+	if ($options->{lwpobj}){
+		my $lwpobj = $options->{lwpobj};
+		if (defined($lwpobj)){
+			my $lwpobjtype = Scalar::Util::blessed($lwpobj);
+			if (defined($lwpobjtype)){
+				$self = $lwpobj;
+				@ISA = ($lwpobjtype);
+			}
+		}
+	}
+	$self = LWP::UserAgent->new unless $self;
+
 	$self->{api_key}	= $options->{key};
 	$self->{api_secret}	= $options->{secret};
-	$self->{rest_uri}	= $options->{rest_uri} || 'http://www.flickr.com/services/rest/';
-	$self->{auth_uri}	= $options->{auth_uri} || 'http://www.flickr.com/services/auth/';
+	$self->{rest_uri}	= $options->{rest_uri} || 'http://api.flickr.com/services/rest/';
+	$self->{auth_uri}	= $options->{auth_uri} || 'http://api.flickr.com/services/auth/';
+	$self->{unicode}	= $options->{unicode} || 0;
 
 	eval {
 		require Compress::Zlib;
@@ -45,6 +61,7 @@ sub sign_args {
 		$sig .= $key . $value;
 	}
 
+	return md5_hex(encode_utf8($sig)) if $self->{unicode};
 	return md5_hex($sig);
 }
 
@@ -76,7 +93,12 @@ sub request_auth_url {
 sub execute_method {
 	my ($self, $method, $args) = @_;
 
-	my $request = new Flickr::API::Request({'method' => $method, 'args' => $args, rest_uri => $self->{rest_uri}});
+	my $request = Flickr::API::Request->new({
+		'method'	=> $method,
+		'args'		=> $args,
+		'rest_uri'	=> $self->{rest_uri},
+		'unicode'	=> $self->{unicode},
+	});
 
 	$self->execute_request($request);
 }
@@ -104,7 +126,10 @@ sub execute_request {
 		return $response;
 	}
 
-	my $tree = XML::Parser::Lite::Tree::instance()->parse($response->decoded_content());
+	my $content = $response->decoded_content();
+	$content = $response->content() unless defined $content;
+
+	my $tree = XML::Parser::Lite::Tree::instance()->parse($content);
 
 	my $rsp_node = $self->_find_tag($tree->{children});
 
@@ -151,8 +176,11 @@ Flickr::API - Perl interface to the Flickr API
 
   use Flickr::API;
 
-  my $api = new Flickr::API({'key'    => 'your_api_key',
-                             'secret' => 'your_app_secret'});
+  my $api = Flickr::API->new({
+		'key'    => 'your_api_key',
+		'secret' => 'your_app_secret',
+		'unicode'=> 0,
+	});
 
   my $response = $api->execute_method('flickr.test.echo', {
 		'foo' => 'bar',
@@ -164,9 +192,9 @@ or
   use Flickr::API;
   use Flickr::API::Request;
 
-  my $api = new Flickr::API({'key' => 'your_api_key'});
+  my $api = Flickr::API->new({'key' => 'your_api_key'});
 
-  my $request = new Flickr::API::Request({
+  my $request = Flickr::API::Request->new({
 		'method' => 'flickr.test.echo',
 		'args' => {},
 	});
@@ -181,38 +209,68 @@ A simple interface for using the Flickr API.
 C<Flickr::API> is a subclass of L<LWP::UserAgent>, so all of the various
 proxy, request limits, caching, etc are available.
 
-=head2 METHODS
+=head1 METHODS
 
-=over 4
+=over
+
+=item C<new({ opt =E<gt> 'value', ... })>
+
+Returns as new L<Flickr::API> object. The options are as follows:
+
+=over
+
+=item C<key> (required)
+
+Your API key
+
+=item C<secret>
+
+Your API key's secret
+
+=item C<rest_uri> & C<auth_uri>
+
+Override the URIs used for contacting the API.
+
+=item C<lwpobj>
+
+Base the C<Flickr::API> on this object, instead of creating a new instance of L<LWP::UserAgent>.
+This is useful for using the features of e.g. L<LWP::UserAgent::Cached>.
+
+=item C<unicode>
+
+This flag controls whether Flicrk::API expects you to pass UTF-8 bytes (unicode=0, the default) or
+actual unicode strings (unicode=1) in the request.
+
+=back
 
 =item C<execute_method($method, $args)>
 
-Constructs a C<Flickr::API::Request> object and executes it, returning a C<Flickr::API::Response> object.
+Constructs a L<Flickr::API::Request> object and executes it, returning a L<Flickr::API::Response> object.
 
 =item C<execute_request($request)>
 
-Executes a C<Flickr::API::Request> object, returning a C<Flickr::API::Response> object. Calls are signed
-if a secret was specified when creating the C<Flickr::API> object.
+Executes a L<Flickr::API::Request> object, returning a L<Flickr::API::Response> object. Calls are signed
+if a secret was specified when creating the L<Flickr::API> object.
 
 =item C<request_auth_url($perms,$frob)>
 
-Returns a C<URI> object representing the URL that an application must redirect a user to for approving
+Returns a L<URI> object representing the URL that an application must redirect a user to for approving
 an authentication token.
 
 For web-based applications I<$frob> is an optional parameter.
 
 Returns undef if a secret was not specified when creating the C<Flickr::API> object.
 
-
 =back
 
-
 =head1 AUTHOR
 
-Copyright (C) 2004-2005, Cal Henderson, E<lt>cal@iamcal.comE<gt>
+Copyright (C) 2004-2013, Cal Henderson, E<lt>cal@iamcal.comE<gt>
 
 Auth API patches provided by Aaron Straup Cope
 
+Subclassing patch from AHP
+
 
 =head1 SEE ALSO
 
@@ -221,5 +279,6 @@ L<Flickr::API::Response>,
 L<XML::Parser::Lite>,
 L<http://www.flickr.com/>,
 L<http://www.flickr.com/services/api/>
+L<https://github.com/iamcal/perl-Flickr-API>
 
 =cut
